from hologram import FieldEncoder, JsonSchemaMixin, ValidationError

import pytest

from dataclasses import dataclass
from typing import NewType, List, Union


ListOrTuple = NewType("ListOrTuple", List[str])


class ListOrTupleEncoder(FieldEncoder):
    def to_wire(self, value):
        return list(value)

    def to_python(self, value):
        return value

    @property
    def json_schema(self):
        return {"type": "array", "items": {"type": "string"}}


class ListOrTupleOrAloneEncoder(FieldEncoder):
    def to_wire(self, value):
        if isinstance(value, (tuple, list)):
            return list(value)
        else:
            return [value]

    def to_python(self, value):
        if isinstance(value, (tuple, list)):
            return list(value)
        else:
            return [value]

    @property
    def json_schema(self):
        return {"type": ["array", "string"], "items": {"type": "string"}}


JsonSchemaMixin.register_field_encoders(
    {
        ListOrTuple: ListOrTupleEncoder(),
        Union[ListOrTuple, str]: ListOrTupleOrAloneEncoder(),
        Union[str, ListOrTuple]: ListOrTupleOrAloneEncoder(),
    }
)


@dataclass
class Foo(JsonSchemaMixin):
    thing: Union[ListOrTuple, str]


def test_registered():
    first = Foo(thing="one")
    second = Foo(thing=["two", "2"])
    third = Foo(thing=("three", "3"))

    assert first.to_dict() == {"thing": ["one"]}
    assert Foo.from_dict({"thing": "one"}) == Foo(thing=["one"])

    assert second.to_dict() == {"thing": ["two", "2"]}
    assert Foo.from_dict({"thing": ["two", "2"]}) == second

    assert third.to_dict() == {"thing": ["three", "3"]}
    # dicts must be valid json
    with pytest.raises(ValidationError):
        Foo.from_dict({"thing": ("three", "3")})



from hologram import JsonSchemaMixin

from dataclasses import dataclass, field
from typing import List, Dict, Any


@dataclass
class Foo(JsonSchemaMixin):
    x: int = 100
    y: List[str] = field(default_factory=list)
    z: Dict[str, Any] = field(default_factory=dict)


def test_basic():
    foo = Foo()
    assert foo.to_dict() == {"x": 100, "y": [], "z": {}}
    assert Foo.from_dict({}) == foo


def test_complex():
    json_obj = {
        "a": 1,
        "b": ["hello", "world!"],
        "c": {"key1": ["value1", "value2"], "key2": "just a string"},
    }
    full_dict = {"x": 10, "y": ["a", "b"], "z": json_obj}
    foo = Foo(10, ["a", "b"], json_obj)
    assert foo.to_dict() == full_dict
    assert Foo.from_dict(full_dict) == foo
